import inspect
from importlib import import_module

from django.conf import settings
from django.urls import path

from fast_drf.core.api_generator import APIGenerator
from fast_drf.utils import predicate
from fast_drf.utils.enums import HTTPVerbsEnum
from fast_drf.utils.parser import get_config


class BasicRouter(object):
    """
    BasicRouter is for making basic urls for django app.
    """

    @classmethod
    def get_urls(cls, **kwargs):
        """
        Making Django Url here.
        :param kwargs:
        :return: a tuple of list [(model_name, model), (model_name, model)]
        """
        _urls = []
        _config = get_config()
        apps = _config.get('DEFAULT_APPLIED_APPS', [])
        for app in apps:
            try:
                _model_tuple = inspect.getmembers(import_module("{0}.{1}".format(app, 'models')), predicate.is_model)
                for model_name, model in _model_tuple:
                    _model_api_config = model.exposed_api() if hasattr(model, 'exposed_api') else {}
                    if not _model_api_config:
                        continue
                    # Updating model into exposed_api returned value/dict
                    _model_api_config.update({'model': model})
                    expose_api_object = APIGenerator(**_model_api_config)
                    # Get API Version information
                    api_versions = model.api_version_fields() if hasattr(model, 'api_version_fields') else {}
                    if len(api_versions.keys()) > 0:
                        for version in api_versions.keys():
                            _viewset_class = expose_api_object.get_runtime_viewset(api_version=version,
                                                                                   **_model_api_config)
                            _urls += [
                                path(
                                    str(cls.get_url_string(value=_model_api_config['api_url'], version=version)),
                                    cls.get_viewset_class_view(view_class=_viewset_class, api_config=_model_api_config),
                                    name=str(_model_api_config['api_url'])
                                )
                            ]
                    else:
                        _viewset_class = expose_api_object.get_runtime_viewset(**_model_api_config)
                        _urls += [
                            path(
                                str(cls.get_url_string(_model_api_config['api_url'], version='v1')),
                                cls.get_viewset_class_view(view_class=_viewset_class, api_config=_model_api_config),
                                name=str(_model_api_config['api_url'])
                            )
                        ]
                    if any([
                        HTTPVerbsEnum.PUT.value in _model_api_config.get('allowed_methods', []),
                        HTTPVerbsEnum.PATCH.value in _model_api_config.get('allowed_methods', []),
                        HTTPVerbsEnum.DELETE.value in _model_api_config.get('allowed_methods', []),
                    ]):
                        if len(api_versions.keys()) > 0:
                            for version in api_versions.keys():
                                _viewset_class = expose_api_object.get_runtime_viewset(api_version=version,
                                                                                       **_model_api_config)
                                _urls += [path(
                                    str(cls.get_url_string(_model_api_config['api_url'],
                                                           version=version)) + '<str:{0}>/'.format(
                                        _model_api_config.get('slug_field', 'pk')),
                                    cls.get_viewset_class_view(view_class=_viewset_class,
                                                               api_config=_model_api_config, details=True),
                                    name=str(_model_api_config['api_url']) + '_details'
                                )]
                        else:
                            _viewset_class = expose_api_object.get_runtime_viewset(**_model_api_config)
                            _urls += [path(
                                str(cls.get_url_string(_model_api_config['api_url'],
                                                       version='v1')) + '<str:{0}>/'.format(
                                    _model_api_config.get('slug_field', 'pk')),
                                cls.get_viewset_class_view(view_class=_viewset_class,
                                                           api_config=_model_api_config, details=True),
                                name=str(_model_api_config['api_url']) + '_details'
                            )]
            except (ModuleNotFoundError,) as error:
                continue
        return _urls

    @classmethod
    def get_url_string(cls, value, version=None):
        _config = get_config()
        if settings.APPEND_SLASH:
            value = "{0}/".format(value)
        if version:
            value = "{prefix}/{version}/{value}".format(
                prefix=_config.get('DEFAULT_API_PREFIX', 'api'), version=version, value=value)
        return value

    @classmethod
    def get_viewset_class_view(cls, view_class, api_config, **kwargs):
        # TODO: I'll rewrite this nonsense try;except statement
        try:
            _allowed_methods = api_config.get('allowed_methods', ['get'])
            actions_dict = {}
            # Default get method from parent function
            details_api = kwargs.get('details', False)
            if details_api:
                actions_dict.update(get='retrieve')
                if HTTPVerbsEnum.PUT.value in _allowed_methods:
                    actions_dict.update(put='update')
                if HTTPVerbsEnum.PATCH.value in _allowed_methods:
                    actions_dict.update(patch='partial_update')
                if HTTPVerbsEnum.DELETE.value in _allowed_methods:
                    actions_dict.update(delete='destroy')
            else:
                actions_dict.update(get='list')
                if HTTPVerbsEnum.POST.value in _allowed_methods:
                    actions_dict.update(post='create')

            return view_class.as_view(actions_dict)
        except Exception:
            return view_class.as_view()



from rest_framework.serializers import ModelSerializer

from fast_drf.tests.test_app.models import Post


class PostPrivateSerializer(ModelSerializer):
    class Meta:
        model = Post
        fields = ('title', 'description',)



from django.contrib.auth.models import User
from django.db import models

from fast_drf.mixins.expose_api_model_mixin import ExposeApiModelMixin
from fast_drf.utils.enums import HTTPVerbsEnum


class UserProfile(ExposeApiModelMixin, models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    name = models.CharField(max_length=128)
    phone = models.CharField(max_length=32)
    address = models.TextField(verbose_name='Address')
    dob = models.DateField(null=True)

    class Meta:
        app_label = 'test_app'

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        api_configs = {
            "api_url": "user-profiles",
        }
        return api_configs


class TestUser(UserProfile):
    class Meta:
        proxy = True
        app_label = 'test_app'

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        api_configs = {
            "api_url": "test-users",
        }
        return api_configs


class PostMeta(models.Model):
    meta_info = models.TextField(null=True, blank=True)

    class Meta:
        app_label = 'test_app'


    @classmethod
    def api_prefetch_related_fields(cls):
        return ['posts']

    @classmethod
    def api_select_related_fields(cls):
        return []



class Post(ExposeApiModelMixin, models.Model):
    author = models.ForeignKey(UserProfile, on_delete=models.SET_NULL, null=True)
    title = models.CharField(max_length=256, null=True)
    description = models.TextField(null=True)
    meta = models.ForeignKey(PostMeta, on_delete=models.SET_NULL, null=True, related_name='posts')

    class Meta:
        app_label = 'test_app'

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        from .views import PostAPIView
        from .serializers import PostPrivateSerializer

        api_configs = {
            "api_url": "posts",
            # "viewset_class": PostAPIView,
            # "serializer_class": PostPrivateSerializer,
            "allowed_methods": [HTTPVerbsEnum.GET.value, HTTPVerbsEnum.POST.value, HTTPVerbsEnum.PUT.value],
            "queryset": cls.objects.filter()
        }
        return api_configs

    @classmethod
    def api_version_fields(cls, **kwargs):
        """
        *** DEFAULT VERSION `v1` ***

        This method will return a dictionary object with version number and fields name. Fields are similar like
        serializer fields. Or you can say exactly as same as serializer fields.
        :param kwargs: Currently nothing to receive on kwargs
        :return: a dictionary object with version number
        """
        versions = {
            'v1': ['id', 'author', 'title', 'description'],
            'v2': ['pk', 'author', 'title', 'description', 'meta']
        }
        return versions

    @classmethod
    def api_prefetch_related_fields(cls):
        return []

    @classmethod
    def api_select_related_fields(cls):
        return ['meta', 'author']




import enum

__author__ = 'Ashraful'


class BaseEnum(enum.Enum):
    pass


class HTTPVerbsEnum(BaseEnum):
    GET = 'get'
    POST = 'post'
    PUT = 'put'
    PATCH = 'patch'
    DELETE = 'delete'



from typing import Any

from django.db import transaction
from django.db.models import ForeignKey, Model, OneToOneField
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.fields import empty


class SerializerGenerator(object):
    def __init__(self, *args, **kwargs):
        self.model = kwargs.get('model')

    def make_runtime_serializer(self, api_version=None, **func_kwargs):
        """
        A generic serializer maker once at a time
        :param func_kwargs: all the extra params are accepted and pass to child
        :param api_version: API version string or number
        :return: return serializer class
        """
        # Define a protected _this to access outer scope
        _this = self
        api_version_fields = self.model.api_version_fields() if hasattr(self.model, 'api_version_fields') else {}
        current_version_fields = api_version_fields.get(api_version, '__all__')

        class RuntimeModelSerializer(serializers.ModelSerializer):
            def __init__(self, instance=None, data=empty, **kwargs):
                if data is not empty:
                    data = self.create_relational_data(data=data)
                self.api_version = api_version
                super(RuntimeModelSerializer, self).__init__(instance=instance, data=data, **kwargs)

            @classmethod
            def get_api_version(cls):
                return api_version

            class Meta:
                model = self.model
                fields = current_version_fields

            def create(self, validated_data: Any):
                with transaction.atomic():
                    instance = super(RuntimeModelSerializer, self).create(validated_data=validated_data)
                    return instance

            def update(self, instance: Model, validated_data: Any):
                with transaction.atomic():
                    instance = super(RuntimeModelSerializer, self).update(
                        instance=instance, validated_data=validated_data)
                    return instance

            def create_relational_data(self, data, **kwargs):
                # Getting the relational fields
                relational_fields = _this.get_relational_fields()
                # Iterate over the list
                for field in relational_fields:
                    if field.name not in data.keys():
                        # If the field is not present on the given data(User inputted data)
                        continue
                    if type(data[field.name]) == dict:
                        _model = field.related_model
                        try:
                            # Creating related data here
                            related_instance = _model.objects.create(**data[field.name])
                            data.pop(field.name)
                        except TypeError:
                            raise ValidationError({'message': _('{0} contains invalid data.'.format(field.name))})
                        # While no error and data has created then assign the PK to the serializer field.
                        data[field.name] = related_instance.pk
                return data

        return RuntimeModelSerializer

    def get_relational_fields(self, **kwargs):
        # Currently we only have support for creating One2One and ForeignKey field support
        _relational_fields = [OneToOneField, ForeignKey]
        _fields = [f for f in self.model._meta.get_fields() if f.__class__ in _relational_fields]
        return _fields



from django.test import TestCase
from .test_app.models import Post
from fast_drf.utils.enums import HTTPVerbsEnum


class FastDRFTestCase(TestCase):

    def setUp(self):
        self.model = Post
        api_config = {
            'api_url': 'posts',
            'allowed_methods': [HTTPVerbsEnum.GET.value, HTTPVerbsEnum.POST.value, HTTPVerbsEnum.PUT.value],
        }

        self.api_config_kwargs = {'model': self.model}
        self.api_config_kwargs.update(**api_config)




from django.db.models.query import QuerySet
from rest_framework.viewsets import ModelViewSet

from fast_drf.core.serializer_generator import SerializerGenerator
from .base import FastDRFTestCase
from ..core.viewset_generator import APIViewSetGenerator


class ViewsetGeneratorTestCase(FastDRFTestCase):
    def setUp(self):
        super(ViewsetGeneratorTestCase, self).setUp()
        self.serializer_generator = SerializerGenerator(**self.api_config_kwargs)
        self.serializer_class = self.serializer_generator.make_runtime_serializer()
        _kwargs = {
            'model': self.model,
            'serializer_class': self.serializer_class,
            'slug_field': 'title'
        }
        self.viewset_generator = APIViewSetGenerator(**_kwargs)

    def test_get_queryset(self):
        _queryset = self.viewset_generator.get_queryset()
        self.assertIsInstance(_queryset, QuerySet)

    def test_generate_viewset_class(self):
        _viewset_class = self.viewset_generator.make_runtime_viewset()
        self.assertTrue(issubclass(_viewset_class, ModelViewSet))
        self.assertEqual(_viewset_class.serializer_class, self.serializer_class)
        self.assertIsInstance(_viewset_class.queryset, QuerySet)
        self.assertEqual(_viewset_class.lookup_field, 'title')



import inspect

__author__ = 'Ashraful'


def is_model(value=None):
    """
    Check a python object is a model or not. Got idea from inspect
    :param value: an object. may be function, class or anything
    :return: boolean status based on condition
    """
    if value is None:
        return False
    if inspect.isclass(value):
        try:
            _obj = value()
            if hasattr(_obj, 'pk'):
                return True
        except Exception:
            return False
    return False



from django.db.models import Q

from fast_drf.tests.base import FastDRFTestCase
from fast_drf.utils.parser import get_default_config, get_config, parse_filters
from fast_drf.utils.predicate import is_model


# Declaring the helper classes those will be used on the tests
# Declare a non model
class TestNonModel:
    foo = 'bar'


# Fake a request class
class FakeRequest:
    def __init__(self, get={}, post={}):
        self.GET = get
        self.POST = post


class TestUtilityMethod(FastDRFTestCase):
    def test_is_model(self):
        self.assertTrue(is_model(self.model))
        self.assertFalse(is_model(TestNonModel))

    def test_get_default_config(self):
        _config = get_default_config()
        self.assertIsInstance(_config, dict)
        self.assertTrue('DEFAULT_APPLIED_APPS' in _config)
        self.assertTrue('DEFAULT_API_PREFIX' in _config)

    def test_get_config(self):
        _config = get_config()
        self.assertIsInstance(_config, dict)
        self.assertTrue('DEFAULT_APPLIED_APPS' in _config)
        self.assertTrue('DEFAULT_API_PREFIX' in _config)

    def test_parse_filters(self):
        _request = FakeRequest(get={'title:icontains': 'Test', 'description': 'Test Passed.'})
        _filters = parse_filters(model=self.model, request=_request)
        self.assertIsInstance(_filters, Q)



from rest_framework.serializers import ModelSerializer

from fast_drf.core.serializer_generator import SerializerGenerator
from .base import FastDRFTestCase


class SerializerGeneratorTestCase(FastDRFTestCase):
    def setUp(self):
        super(SerializerGeneratorTestCase, self).setUp()
        self.serializer_generator = SerializerGenerator(**self.api_config_kwargs)

    def test_generate_serializer_class(self):
        _serializer_class = self.serializer_generator.make_runtime_serializer()
        self.assertTrue(issubclass(_serializer_class, ModelSerializer))
        self.assertTrue(hasattr(_serializer_class, 'Meta'))
        self.assertEqual(getattr(_serializer_class.Meta, 'model'), self.model)

    def test_serializer_versioned_fields(self):
        # Test for version v1
        version_1_serializer_class = self.serializer_generator.make_runtime_serializer(api_version='v1')
        self.assertTrue(issubclass(version_1_serializer_class, ModelSerializer))
        self.assertTrue(hasattr(version_1_serializer_class, 'Meta'))
        self.assertEqual(getattr(version_1_serializer_class.Meta, 'model'), self.model)
        self.assertEqual(self.model.api_version_fields()['v1'], version_1_serializer_class.Meta.fields)
        # Test for version v2
        version_2_serializer_class = self.serializer_generator.make_runtime_serializer(api_version='v2')
        self.assertTrue(issubclass(version_2_serializer_class, ModelSerializer))
        self.assertTrue(hasattr(version_2_serializer_class, 'Meta'))
        self.assertEqual(getattr(version_2_serializer_class.Meta, 'model'), self.model)
        self.assertEqual(self.model.api_version_fields()['v2'], version_2_serializer_class.Meta.fields)

    def test_get_relational_fields(self):
        fields = self.serializer_generator.get_relational_fields(**self.api_config_kwargs)
        self.assertTrue(isinstance(fields, list))



from fast_drf.core.serializer_generator import SerializerGenerator
from fast_drf.core.viewset_generator import APIViewSetGenerator


class APIGenerator(object):
    def __init__(self, *args, **kwargs):
        self.api_view = APIViewSetGenerator(**kwargs)
        self.serializer = SerializerGenerator(**kwargs)

    def get_runtime_viewset(self, api_version=None, **kwargs):
        """
        Viewset finder and return
        :param kwargs: all the extra params are accepted and pass to child
        :param api_version: API version string or no
        :return: a viewset class
        """
        # If not serializer is present
        if not self.api_view.serializer_class or (
                self.api_view.viewset_class and not self.api_view.viewset_class.serializer_class) or (
                self.api_view.serializer_class and (
                hasattr(self.api_view.serializer_class, 'get_api_version')
                and self.api_view.serializer_class.get_api_version() != api_version)):
            _serializer_class = self.get_runtime_serializer(api_version=api_version, **kwargs)
            self.api_view.serializer_class = _serializer_class
            if self.api_view.viewset_class:
                self.api_view.viewset_class.serializer_class = _serializer_class
            # Now serializer is available
        if self.api_view.viewset_class is None:
            return self.api_view.make_runtime_viewset(**kwargs)
        return self.api_view.viewset_class

    def get_runtime_serializer(self, api_version=None, **kwargs):
        """
        Serializer finder and return
        :param kwargs: all the extra params are accepted and pass to child
        :param api_version: API Version string or number
        :return: return a serializer class
        """
        if (self.api_view.serializer_class is None and self.api_view.model) or (
                self.api_view.serializer_class and (
                hasattr(self.api_view.serializer_class, 'get_api_version')
                and self.api_view.serializer_class.get_api_version() != api_version)):
            return self.serializer.make_runtime_serializer(api_version=api_version, **kwargs)
        return self.api_view.serializer_class



class ExposeApiModelMixin(object):
    """
    ==== MODEL MIXIN ====

    This Mixin(Class) will help to generate automatic api from models with few configs
    """

    @classmethod
    def exposed_api(cls, *args, **kwargs):
        """
        This method holds a bunch of API configs and return like following...
        {
            "api_url": "",  # (REQUIRED)

            # You must use from HTTPVerbsEnum. Like HTTPVerbsEnum.GET.value, HTTPVerbsEnum.POST.value
            "allowed_methods": ['get', 'post', 'put', 'patch', 'delete'], # (NOT REQUIRED)

            # slug_field is application 'put', 'patch', 'delete' these methods
            "slug_field": "pk", # (NOT REQUIRED) DEFAULT [PK] (Must be model field, unique or primary key)

            "queryset": "",  # (NOT REQUIRED) default all
            "viewset_class": "",  # (NOT REQUIRED) BaseViewset class
            "serializer_class": "",  # (NOT REQUIRED) default BaseEntitySerializer
            "permission_classes": "",  # (NOT REQUIRED) default set from settings
        }
        :param args:
        :param kwargs:
        :return: An empty Dictionary/False OR Full config dictionary.
        """
        api_configs = {}
        return api_configs

    @classmethod
    def api_version_fields(cls, **kwargs):
        """
        *** DEFAULT VERSION `v1` ***

        This method will return a dictionary object with version number and fields name. Fields are similar like
        serializer fields. Or you can say exactly as same as serializer fields.
        :param kwargs: Currently nothing to receive on kwargs
        :return: a dictionary object with version number
        """
        versions = {}
        return versions

    @classmethod
    def api_prefetch_related_fields(cls):
        """
        Prefetch related is used for prefeching the objects for Django Queryset. It saves a lot time
        of table joining and multiple queries. 
        Read more: https://docs.djangoproject.com/en/3.1/ref/models/querysets/#prefetch-related

        Return a lists of field name those will be directly using on the queryset.
        """
        return []

    @classmethod
    def api_select_related_fields(cls):

        """
        Select related is used for prefeching the objects for Django Queryset. It saves a lot time
        of table joining and multiple queries. 
        Read more: https://docs.djangoproject.com/en/3.1/ref/models/querysets/#select-related

        Return a lists of field name those will be directly using on the queryset.
        """
        return []



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='fast-drf',
    version='2.1.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A simple model based API configarator written in Python and based of Django and Django REST Framework',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://ashraful.dev',
    author='Ashraful Islam',
    author_email='ashrafulrobin3@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'django>=2.0',
        'djangorestframework>=3.10',
    ]
)



from rest_framework import viewsets
from rest_framework.response import Response

from fast_drf.utils.parser import parse_filters

__author__ = 'Ashraful'



class APIViewSetGenerator(object):
    permission_classes = None
    serializer_class = None
    viewset_class = None
    queryset = None
    model = None

    def __init__(self, *args, **main_kwargs):
        self.model = main_kwargs.get('model')
        self.serializer_class = main_kwargs.get('serializer_class', None)
        self.viewset_class = main_kwargs.get('viewset_class')
        self.permission_classes = main_kwargs.get('permission_classes', [])
        self.queryset = main_kwargs.get('queryset', self.get_queryset(**main_kwargs))
        self.lookup_field = main_kwargs.get('slug_field', 'pk')

    def get_queryset(self, *args, **kwargs):
        """
        Queryset maker for model
        :param args:
        :param kwargs:
        :return: a queryset
        """
        return self.model.objects.all()

    def make_runtime_viewset(self, **kwargs):
        """
        Viewset Maker while it's not available on model. It's a generic viewset maker for any model
        :param func_kwargs: all the extra params are accepted and pass to child
        :return: return a viewset
        """

        class RunTimeViewset(viewsets.ModelViewSet):
            if self.permission_classes:
                permission_classes = self.permission_classes
            serializer_class = self.serializer_class
            queryset = self.queryset
            model = self.model
            lookup_field = self.lookup_field

            def get_queryset(self, *args, **kwargs):
                # Here it's performing sub query in SQL. So, no performance loss. Just executing a big query
                # not more than 1 query
                _prefetch_related_fields = self.model.api_prefetch_related_fields()
                _select_related_fields = self.model.api_select_related_fields()
                queryset = self.model.objects.filter(pk__in=self.queryset)
                if len(_prefetch_related_fields) > 0:
                    queryset = queryset.prefetch_related(*_prefetch_related_fields)
                if len(_select_related_fields) > 0:
                    queryset = queryset.select_related(*_select_related_fields)
                return queryset

            def list(self, request, **kwargs):
                base_queryset = self.get_queryset(request=request, **kwargs)
                request = kwargs.get('request', self.request)
                try:
                    search_enabled = bool(eval(request.GET.get('search', '0')))
                    if search_enabled:
                        _filters = parse_filters(model=self.model, request=request)
                        self.queryset = base_queryset.filter(_filters)
                except Exception as err:
                    self.queryset = base_queryset
                return super(RunTimeViewset, self).list(request=request, **kwargs)

            def create(self, request, *args, **kwargs):
                serializer = self.serializer_class(data=request.data)
                if serializer.is_valid(raise_exception=True):
                    serializer.save()
                    return Response(serializer.data)
                return Response(serializer.data)

            def retrieve(self, request, *args, **kwargs):
                return super(RunTimeViewset, self).retrieve(request, *args, **kwargs)

            def update(self, request, *args, **kwargs):
                return super(RunTimeViewset, self).update(request, *args, **kwargs)

            def partial_update(self, request, pk=None, *args, **kwargs):
                return super(RunTimeViewset, self).partial_update(request=request, pk=pk, *args, **kwargs)

            def destroy(self, request, pk=None, *args, **kwargs):
                return super(RunTimeViewset, self).destroy(request=request, pk=pk, *args, **kwargs)

        return RunTimeViewset



# Generated by Django 3.1.7 on 2021-03-21 09:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import fast_drf.mixins.expose_api_model_mixin


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PostMeta',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('meta_info', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('phone', models.CharField(max_length=32)),
                ('address', models.TextField(verbose_name='Address')),
                ('dob', models.DateField(null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            bases=(fast_drf.mixins.expose_api_model_mixin.ExposeApiModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=256, null=True)),
                ('description', models.TextField(null=True)),
                ('author', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='test_app.userprofile')),
                ('meta', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='posts', to='test_app.postmeta')),
            ],
            bases=(fast_drf.mixins.expose_api_model_mixin.ExposeApiModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TestUser',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('test_app.userprofile',),
        ),
    ]



from copy import deepcopy


def get_default_config():
    """
    Here will be landed all the default config.
    """
    from django.conf import settings

    return {
        'DEFAULT_APPLIED_APPS': settings.INSTALLED_APPS,
        'DEFAULT_API_PREFIX': 'api',
    }


def get_config():
    """
    A little business logic is applied here. It'll always apply the user config with default config
    """
    from django.conf import settings
    _settings_key = 'FAST_DRF_CONFIG'
    default_config = get_default_config()

    if hasattr(settings, _settings_key):
        _config = getattr(settings, _settings_key, {})
        default_config.update(_config)
    return default_config


def parse_filters(model, request, **kwargs):
    from django.db.models import Q
    """
    model: The Django model,
    request: The request where user asked for the data
    """
    _fields = [f.name for f in model._meta.get_fields()]
    _params = request.GET
    _filters = {}
    for param, val in _params.items():
        try:
            if ':' in param:
                _field_name, _filter_option = param.split(':')
                if _field_name not in _fields:
                    continue
                _temp = deepcopy(param)
                _filters['{}__{}'.format(_field_name, _filter_option)] = val
            else:
                if param not in _fields:
                    continue
                _filters[param] = val
        except Exception as error:
            continue
    return Q(**_filters)


